from datetime import datetime as dt, timedelta as td
from django.test import TestCase
from django.contrib.auth import authenticate, login as dj_login
from django.contrib.auth.models import User
from zeropass.models import Token
from zeropass.backends import TokenBackend


class BackendTestCases(TestCase):

    def test_get_user_exist(self):
        """get_user() returns a user"""
        u = User()
        u.is_active = True
        u.email = 'me@example.com'
        u.save()
        # make backend
        tb = TokenBackend()
        user = tb.get_user(u.id)
        # user is same as u
        self.assertEqual(user.id, u.id)

    def test_get_user_not_exist(self):
        """get_user() returns a user"""
        # make backend
        tb = TokenBackend()
        user = tb.get_user(8)
        # user is None
        self.assertEqual(user, None)

    def test_no_match(self):
        """If no match is found then None is returned"""
        # make a fake token
        fake_token = 'a' * 40
        user = authenticate(token=fake_token)
        # user is None
        self.assertEqual(None, user)

    def test_no_match_expired(self):
        """if a token is expired no match is return"""
        # make a user
        u = User()
        u.is_active = True
        u.email = 'me@example.com'
        u.save()
        # make a token with expires in past
        t = Token()
        t.expires = dt.now() - td(seconds=300)
        t.user = u
        t.save()
        # send te tokem through the backend and get user back
        user = authenticate(token=t.token)
        # user is None
        self.assertEqual(user, None)

    def test_match(self):
        """the backend returns the user when a match is found"""
        # make a user
        u = User()
        u.is_active = True
        u.email = 'me@example.com'
        u.save()
        # make a token
        t = Token()
        t.user = u
        t.save()
        # send te tokem through the backend and get user back
        user = authenticate(token=t.token)
        # user the save as above
        self.assertEqual(user.pk, u.pk)



from datetime import datetime as dt
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from zeropass.models import Token


class ManagerTestCases(TestCase):

    def test_expired(self):
        """The expired manager returns expired tokens"""
        # create a user
        u = User()
        u.is_active = True
        u.save()
        # create some tokens
        t1 = Token()
        t1.user = u
        t1.expires = dt(2011, 1, 1, 1, 1, 1)
        t1.save()
        t2 = Token()
        t2.user = u
        t2.expires = dt(2011, 1, 1, 1, 1, 1)
        t2.save()
        # expired returns 2
        self.assertEqual(2, Token.objects.expired().count())
        # update t2
        t2.expires = dt(2013, 1, 1, 1, 1, 1)
        t2.save()
        # now we have only one expired
        self.assertEqual(1, Token.objects.expired().count())


class ModelTestCases(TestCase):

    def test_token_when_saved(self):
        """when a token is saved it get's a random token"""
        u = User()
        u.is_active = True
        u.save()
        # make a token
        t = Token()
        t.user = u
        t.save()
        # the token now has a 40 digit token
        self.assertEqual(40, len(t.token))

    def test_expires_when_save(self):
        """when a token is saved it get's a random token"""
        u = User()
        u.is_active = True
        u.save()
        # make a token
        t = Token()
        t.user = u
        # the token has no
        self.assertFalse(t.expires)
        # save it
        t.save()
        # the token now has an expiry
        self.assertTrue(t.expires)



from django.contrib import admin
from zeropass.models import Token


class TokenAdmin(admin.ModelAdmin):
    raw_id_fields = ('user',)
    list_display = (
        'token',
        'expires',
        'user')


admin.site.register(Token, TokenAdmin)



from django.test import TestCase
from django.core import mail
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from zeropass.models import Token


class ReceiverTestCaseEmail(TestCase):

    def test_email_sends_when_token_is_created(self):
        """when a token is created an email is sent"""
        # create a user
        u = User()
        u.is_active = True
        u.email = 'me@example.com'
        u.save()
        # create a token
        t = Token()
        t.user = u
        t.save()
        # now there is one message
        self.assertEqual(1, len(mail.outbox))



from datetime import datetime as dt, timedelta as td
from unittest import skipUnless
from django.db.models import signals
from django.test import TestCase
from django.test.utils import override_settings
from django.core.management import call_command
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from zeropass.models import Token
from zeropass import receivers


class CleanupTestCases(TestCase):
    """make sure management command and celery task
    are doing their jobs"""

    def test_command(self):
        """the management command deletes expired tokens"""
        # create a user
        u = User()
        u.is_active = True
        u.save()
        # create some tokens
        t1 = Token()
        t1.user = u
        t1.expires = dt(2011, 1, 1, 1, 1, 1)
        t1.save()
        t2 = Token()
        t2.user = u
        t2.expires = dt(2011, 1, 1, 1, 1, 1)
        t2.save()
        t3 = Token()
        t3.user = u
        t3.expires = dt(2013, 1, 1, 1, 1, 1)
        t3.save()
        # call the managemet command
        call_command('zeropass_cleanup')
        # we have none
        self.assertEqual(1, Token.objects.count())



from django.core.mail import EmailMultiAlternatives
from django.conf import settings
from django.contrib.sites.models import Site
from django.template import loader, Context
from zeropass import conf


def send_email(sender, instance, **kwargs):
    """send the email to the person who requested the token"""
    #
    #
    # the site
    site = Site.objects.get_current()
    #
    #
    # the subjects
    subject = kwargs.get('subject') or '%s login' % site.name
    #
    #
    # the context
    c = Context({
        'token': instance,
        'site': site,
        'protocol': conf.PROTOCOL
    })
    #
    #
    # generate the text content
    t = loader.get_template('zeropass/email.txt')
    text_content = t.render(c)
    #
    #
    # build the message
    msg = EmailMultiAlternatives(
        subject,
        text_content,
        settings.DEFAULT_FROM_EMAIL,
        [instance.user.email])
    #
    #
    # if we're sending HTML
    if conf.HTML_EMAILS:
        #
        #
        # generate the html content
        t = loader.get_template('zeropass/email.html')
        html_content = t.render(c)
        msg.attach_alternative(html_content, "text/html")
    #
    #
    # now send it
    msg.send()



from datetime import datetime
from django.contrib.auth.models import User
from zeropass.models import Token


class TokenBackend(object):

    def authenticate(self, **kwargs):
        try:
            # get the token
            token = Token.objects.get(
                token=kwargs['token'],
                expires__gt=datetime.now())
            # get the user
            return token.user
        except (KeyError, Token.DoesNotExist):
            return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



import os
ROOT = os.path.dirname(os.path.abspath(__file__))
DEBUG = True
TEMPLATE_DEBUG = DEBUG
ADMINS = ()
MANAGERS = ADMINS
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'zeropass',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

TIME_ZONE = 'America/Chicago'
LANGUAGE_CODE = 'en-us'
SITE_ID = 1
USE_I18N = False
USE_L10N = False
USE_TZ = False
MEDIA_ROOT = ''
MEDIA_URL = ''
STATIC_ROOT = ''
STATIC_URL = '/static/'
STATICFILES_DIRS = ()
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)
SECRET_KEY = 'tc52j+8vrb2o&amp;3-lczk-&amp;utp(pz)%4ez71=*wu4^gp1h_8y2a4'
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'example.urls'
WSGI_APPLICATION = 'example.wsgi.application'
TEMPLATE_DIRS = (
   '%s/templates/' % ROOT
)
print TEMPLATE_DIRS
AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    'zeropass.backends.TokenBackend',
)
INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'zeropass',
)
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



from django.shortcuts import render
from django.core.urlresolvers import reverse
from django.contrib.auth import authenticate, login as dj_login
from django.http import HttpResponseRedirect, Http404
from django.conf import settings
from zeropass.models import Token
from zeropass.forms import TokenForm


def login(request):
    """processes or displays a form depending on
    whether or not post data is present"""

    form = TokenForm(request.POST or None)

    if form.is_valid():
        token = form.save()
        return HttpResponseRedirect(
            reverse('zeropass:sent'))

    return render(
        request,
        'zeropass/login.html',
        {'form': form})


def sent(request):
    """display a nice done type page"""
    return render(
        request,
        'zeropass/sent.html')


def validate(request, token):
    """validates a token and if it is valid it will
    login the user in"""
    # get the token or 404
    try:
        t = Token.objects.get(token=token)
    except Token.DoesNotExist:
        raise Http404('not matching token could be found')
    # attempt to get the user
    user = authenticate(token=token)
    # if we had a user returned
    if user is not None:
        # if user is active log in and redirect if token next
        if user.is_active:
            # log te user in
            dj_login(request, user)
            # set up redirect
            url = t.next or settings.LOGIN_REDIRECT_URL
            # delete the token
            t.delete()
            # return the redirect
            return HttpResponseRedirect(url)
    # redirect to login
    return HttpResponseRedirect(reverse('zeropass:login'))



from django.core.management.base import BaseCommand, CommandError
from zeropass.models import Token


class Command(BaseCommand):
    help = 'deletes all expire zeropass tokens'

    def handle(self, *args, **options):
        expired_tokens = Token.objects.expired()
        expired_tokens.delete()



from string import letters
from datetime import datetime
from hashlib import sha1
from random import choice
from datetime import datetime as dt, timedelta as td
from django.db import models
from django.contrib.auth.models import User
from zeropass import conf
from zeropass import receivers


class TokenManager(models.Manager):

    def expired(self):
        return super(TokenManager, self).get_query_set()\
            .filter(expires__lt=datetime.now())


class Token(models.Model):
    user = models.ForeignKey(User)
    token = models.CharField(
        blank=True,
        null=True,
        max_length=40)
    expires = models.DateTimeField()
    next = models.CharField(
        max_length=255,
        blank=True,
        null=True)

    objects = TokenManager()

    def __unicode__(self):
        return '<Token for %s>' % self.user.username

    def save(self, *args, **kwargs):
        # set the expires
        if self.expires is None:
            self.expires = dt.now() + td(seconds=conf.EXPIRES)
        # set the token
        if self.token is None or self.token == '':
            seed = ''.join(choice(letters) for x in range(10))
            self.token = sha1(seed).hexdigest()
        super(Token, self).save(*args, **kwargs)
#
#
# connect the receivers
models.signals.post_save.connect(
    receivers.send_email,
    sender=Token,
    dispatch_uid='receivers_send_email')



import hashlib
from datetime import datetime, timedelta
from django import forms
from django.contrib.auth.models import User
from zeropass.models import Token
from zeropass import conf


class TokenForm(forms.Form):

    email = forms.EmailField()

    def clean_email(self):
        email = self.cleaned_data['email']
        try:
            self.user = User.objects.get(email=email)
        except User.DoesNotExist:
            raise forms.ValidationError('Email not found')
        return email

    def save(self):
        token = Token()
        token.user = self.user
        token.expires = datetime.now() + timedelta(seconds=conf.EXPIRES)
        token.save()



from django.http import HttpResponse
from django.conf.urls import patterns, include, url
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    (r'^$', lambda x: HttpResponse('')),
    url(r'^auth/', include('zeropass.urls', namespace='zeropass')),
    url(r'^admin/', include(admin.site.urls)),
)



import os
from distutils.core import setup
from zeropass import get_version

root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)

setup(
    name='django-zeropass',
    version=get_version(),
    license='BSD 3 Clause',
    description='An app to allow password free logins',
    long_description=open('README.md').read(),
    author='Jamie Curle',
    author_email='jamie@obscuremetaphor.co.uk',
    url='https://django-zeropass.obscuremetaphor.co.uk',
    package_data={
        'zeropass' : [
            'templates/zeropass/*.html',
            'templates/zeropass/*.txt',
        ],
        'zeropass_example': [
            'example/templates/*.html',
        ]
    },
    packages=[
        'zeropass',
        'zeropass.templatetags',
        'zeropass.management',
        'zeropass.management.commands',
        'zeropass.tests',
        'zeropass_example',
        'zeropass_example.example',
    ],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Framework :: Django',],
    install_requires=[
        'django >= 1.4'
    ])



from django.conf import settings as dj_settings


EXPIRES = getattr(
    dj_settings,
    'ZP_EXPIRES',
    600)

PROTOCOL = getattr(
    dj_settings,
    'ZP_PROTOCOL',
    'https')

HTML_EMAILS = getattr(
    dj_settings,
    'ZP_HTML_EMAILS',
    True)



from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from zeropass.models import Token


class ViewTestCases(TestCase):

    def test_sent_200_get(self):
        """the sent returns 200 on get"""
        r = self.client.get(reverse('zeropass:sent'))
        # 200
        self.assertEqual(200, r.status_code)

    def test_login_200_get(self):
        """the logn returns 200 on get"""
        r = self.client.get(reverse('zeropass:login'))
        # 200
        self.assertEqual(200, r.status_code)

    def test_login_200_post(self):
        """the logn returns 200 on post"""
        r = self.client.post(reverse('zeropass:login'), {})
        # 200
        self.assertEqual(200, r.status_code)

    def test_login_200_post_invalid(self):
        """the logn returns a 302  on posti when valid date is sent"""
        # we have no tokens
        self.assertEqual(0, Token.objects.count())
        # send valid date
        r = self.client.post(
            reverse('zeropass:login'),
            {'email': 'me@example.com'})
        # 200
        self.assertEqual(200, r.status_code)
        # we now have no token
        self.assertEqual(0, Token.objects.count())

    def test_login_200_post_valid(self):
        """the logn returns a 302  on posti when valid date is sent"""
        # create a user
        u = User()
        u.is_active = True
        u.email = 'me@example.com'
        u.save()
        # we have no tokens
        self.assertEqual(0, Token.objects.count())
        # send valid date
        r = self.client.post(
            reverse('zeropass:login'),
            {'email': 'me@example.com'})
        # 302
        self.assertEqual(302, r.status_code)
        # we now have one token
        self.assertEqual(1, Token.objects.count())

    def test_validate_user_has_to_be_active(self):
        """if a user is not active they are redirected to login"""
        u = User()
        u.is_active = False
        u.email = 'me@example.com'
        u.save()
        # make token
        t = Token()
        t.next = '/'
        t.user = u
        t.save()
        # call view and we're logged in
        r = self.client.get(reverse('zeropass:validate', args=[t.token]))
        # we were redirect to '/'
        self.assertRedirects(r, reverse('zeropass:login'))

    def test_validate_valid(self):
        """when validate is passed a valid token a user is logged in"""
        # make user
        u = User()
        u.is_active = True
        u.email = 'me@example.com'
        u.save()
        # make token
        t = Token()
        t.next = '/'
        t.user = u
        t.save()
        # call view and we're logged in
        r = self.client.get(reverse('zeropass:validate', args=[t.token]))
        # we were redirect to '/'
        self.assertRedirects(r, '/')

    def test_validate_invalid_token(self):
        # invalid tokens results in a 404
        r = self.client.get(
            reverse('zeropass:validate', args=['a' * 40]))
        # we have a 404
        self.assertEqual(404, r.status_code)



from django.conf.urls import patterns, include, url


urlpatterns = patterns(
    '',
    url(
        r'^$',
        'zeropass.views.login',
        name='login'),
    url(
        r'^(?P<token>\w{40})',
        'zeropass.views.validate',
        name='validate'),
    url(
        r'^sent/$',
        'zeropass.views.sent',
        name='sent'),
)
